#g.py
#Copyright (C) 2004 Free Software Foundation
#see GPL.txt for license.

#This file contains everything that needs to be accessed by most or all files.


#Dragon Hunt 3.28. Released under the GPL. Copyright 2004
#(Note to self: when changing version, change variables.txt, README.txt,
# and setup.py.)

from Tkinter import *
import ImageTk
#needed for die_roll
from random import random
#needed for save/load game
import pickle
from os import path, mkdir, remove, walk

#needed for scripting
from scripting import *


#This will be displayed on the Title screen and Main screen.
game_name = ""

#general character stats
name = ""
hp = 0
ep = 0
maxhp = 0
maxep = 0
attack = 0
defense = 0
gold = 0
exp = 0
level = 0
skillpoints = 1

#stores attack/defense, adjusted by equipment.
#kept correct by main.refreshmap(). Use for calculations.
adj_attack = 0
adj_defense = 0

#equipment:
#equip[0]=weapon, [1]=armor, [2]=shield
#[3]=helmet, [4]=gloves, [5]=boots
equip = []
for x in range(6):
	equip.append(x)

#inventory: 10 spaces.
inv = []
for x in range(10):
	inv.append(x)

#takes the index in the item[] array, and returns the index
#of the first occurance in the inv[] array
def find_inv_item(num):
	for i in range(len(inv)):
		if inv[i] == num:
			return i
	return -1

#takes the index of an item in the item[] array, and places it
#in the first empty spot in the inv[] array. Retuns the
#index of inv[] item is placed at, or -1 for failure.
def take_inv_item(num):
	for i in range(len(inv)):
		if inv[i] == -1:
			inv[i] = num
			return i
	return -1

#takes the index of an item in the inv[] array,
#removes it, then removes the empty space.
def drop_inv_item(num):
	inv[num] = -1
	for i in range(num+1, len(inv)):
		inv[i-1]=inv[i]
	inv[len(inv)-1] = -1

#default tile width. Set in variables.txt, or default to 32.
tilesize = 32

#location on map: x, y, z coordinate. Map names are given a zgrid on loading.
xgrid = 0
ygrid = 0
zgrid = 1

#controls timed effects such as healing. Loops around every 60 turns (0-59);
#use % command to control.
timestep = 0

#dictionary of script variables.
var_list = {}

#current monster hp. Used to allow activation
#of bombs from inv.
cur_mon_hp = 0

#Name of current hero picture.
cur_hero = "people/hero_w.png"

#Should the player be allowed to move? Used with the move scripting command,
#to prevent moving again after moving.
allow_move = 1

#width of the hp/ep bars
hpbar_width = 0

#current module directory
mod_directory = ""

#Has tk been inited yet? Used between rpg.py and new_game.py to decide where
#to init tk.
init_tk_already = 0

#save the game in saves/input.
#uses pickle, first line is a version number.
def savegame(save_file):
	#If there is no save directory, make one.
	if path.isdir(g.mod_directory + "/saves") == 0:
		if path.exists(g.mod_directory + "/saves") == 1:
			remove(g.mod_directory + "/saves")
		mkdir(g.mod_directory + "/saves")
	save_loc = g.mod_directory + "/saves/" + save_file
	savefile=open(save_loc, 'w')
	#savefile version; update whenever the data saved changes.
	pickle.dump("dh3.2", savefile)
	pickle.dump(name, savefile)
	pickle.dump(hp, savefile)
	pickle.dump(ep, savefile)
	pickle.dump(maxhp, savefile)
	pickle.dump(maxep, savefile)
	pickle.dump(attack, savefile)
	pickle.dump(defense, savefile)
	pickle.dump(gold, savefile)
	pickle.dump(exp, savefile)
	pickle.dump(level, savefile)
	pickle.dump(skillpoints, savefile)

	#equipment is stored by name to increase savefile compatability.
	pickle.dump(len(equip), savefile)
	for i in range(len(equip)):
		pickle.dump(item[equip[i]][0], savefile)
	pickle.dump(len(inv), savefile)
	for i in range(len(inv)):
		pickle.dump(item[inv[i]][0], savefile)
	pickle.dump(xgrid, savefile)
	pickle.dump(ygrid, savefile)
	pickle.dump(zgrid, savefile)
	#skills are stored by name as well.
	num = 0
	for i in range(len(skill)):
		if skill[i][5] == 1:
			num += 1
	pickle.dump(num, savefile)
	for i in range(len(skill)):
		if skill[i][5] == 1:
			pickle.dump(skill[i][0], savefile)

	pickle.dump(timestep, savefile)
	pickle.dump(cur_mon_hp, savefile)
	pickle.dump(var_list, savefile)
	savefile.close()

def loadgame(save_file):
	save_loc = g.mod_directory + "/saves/" + save_file
	savefile=open(save_loc, 'r')
	version = pickle.load(savefile)
	if version != "dh3.2":
		return loadgame_31(save_file)
	global name;		name = pickle.load(savefile)
	global hp;			hp = pickle.load(savefile)
	global ep;			ep = pickle.load(savefile)
	global maxhp;		maxhp = pickle.load(savefile)
	global maxep;		maxep = pickle.load(savefile)
	global attack;		attack = pickle.load(savefile)
	global defense;		defense = pickle.load(savefile)
	global gold;		gold = pickle.load(savefile)
	global exp;			exp = pickle.load(savefile)
	global level;		level = pickle.load(savefile)
	global skillpoints;	skillpoints = pickle.load(savefile)

	#equipment is stored by name to increase savefile compatability.
	global equip;		equip_len = pickle.load(savefile)
	for i in range(equip_len):
		equip[i] = finditem(pickle.load(savefile))
		if item[equip[i]][0] == "Ignore":
			equip[i] = -1
	global inv;			inv_len = pickle.load(savefile)
	for i in range(inv_len):
		inv[i] = finditem(pickle.load(savefile))
		if item[inv[i]][0] == "Ignore":
			inv[i] = -1
	global xgrid;		xgrid = pickle.load(savefile)
	global ygrid;		ygrid = pickle.load(savefile)
	global zgrid;		zgrid = pickle.load(savefile)
	global skill;		skill_len = pickle.load(savefile)
	for i in range(skill_len):
		skill[findskill(pickle.load(savefile))][5] = 1
	global timestep;	timestep = pickle.load(savefile)
	global cur_mon_hp;	cur_mon_hp = pickle.load(savefile)
	global var_list;	var_list = pickle.load(savefile)
	savefile.close()

def loadgame_31(save_file):
	save_loc = g.mod_directory + "/saves/" + save_file
	savefile=open(save_loc, 'r')
	version = pickle.load(savefile)
	if version != "dh3.1":
		print "old savefile version"
		return -1
	global name;		name = pickle.load(savefile)
	global hp;			hp = pickle.load(savefile)
	global ep;			ep = pickle.load(savefile)
	global maxhp;		maxhp = pickle.load(savefile)
	global maxep;		maxep = pickle.load(savefile)
	global attack;		attack = pickle.load(savefile)
	global defense;		defense = pickle.load(savefile)
	global gold;		gold = pickle.load(savefile)
	global exp;			exp = pickle.load(savefile)
	global level;		level = pickle.load(savefile)
	global skillpoints;	skillpoints = pickle.load(savefile)

	#equipment is stored by name to increase savefile compatability.
	global equip;		equip_len = pickle.load(savefile)
	for i in range(equip_len):
		equip[i] = finditem(pickle.load(savefile))
		if item[equip[i]][0] == "Ignore":
			equip[i] = -1
	global inv;			inv_len = pickle.load(savefile)
	for i in range(inv_len):
		inv[i] = finditem(pickle.load(savefile))
		if item[inv[i]][0] == "Ignore":
			inv[i] = -1
	global xgrid;		xgrid = pickle.load(savefile)
	global ygrid;		ygrid = pickle.load(savefile)
	global zgrid;		zgrid = pickle.load(savefile)
	global skill;		skill_len = pickle.load(savefile)
	for i in range(skill_len):
		skill[findskill(pickle.load(savefile))][5] = 1
	global timestep;	timestep = pickle.load(savefile)
	global cur_mon_hp;	cur_mon_hp = pickle.load(savefile)
	savefile.close()

#this calls scripting.py to read the datafiles.
def init_data():
	read_maps()
	read_scripts()
	read_items()
	read_monster()
	global game_name
	game_name = read_game_name()
	read_variables()
	read_shops()

#Get the name of the module from the variables.txt file. Note that this
#does not set the game name, only returns it. This lets it be used in rpg.py
def read_game_name():
	file = open(mod_directory + "/data/variables.txt", 'r')
	line = file.readline()
	while (line != ''):
		line = line.strip()
		if (line[:1] == "#" or line[:1] == ""):
			line = file.readline()
			continue
		if (line.split("=", 1)[0].strip() == "game_name"):
			return line.split("=", 1)[1].strip()
		line = file.readline()

#What dice to roll when starting a new game. 2d array.
new_game_dice = []

def read_variables():
	for i in range(5):
		new_game_dice.append([])
	file = open(mod_directory + "/data/variables.txt", 'r')
	line = file.readline()
	while (line != ''):
		line = line.strip()
		if (line[:1] == "#" or line[:1] == ""):
			line = file.readline()
			continue
		if (line.split("=", 1)[0].strip() == "hp"):
			read_dice(0, line.split("=", 1)[1].strip())
		elif (line.split("=", 1)[0].strip() == "ep"):
			read_dice(1, line.split("=", 1)[1].strip())
		elif (line.split("=", 1)[0].strip() == "attack"):
			read_dice(2, line.split("=", 1)[1].strip())
		elif (line.split("=", 1)[0].strip() == "defense"):
			read_dice(3, line.split("=", 1)[1].strip())
		elif (line.split("=", 1)[0].strip() == "gold"):
			read_dice(4, line.split("=", 1)[1].strip())
		elif (line.split("=", 1)[0].strip() == "tilesize"):
			global tilesize
			tilesize = int(line.split("=", 1)[1].strip())
		line = file.readline()

#given a dice set and string of the form 2d4+5, place in new_game_dice
def read_dice(variable, dice_string):
	first = dice_string.split("d", 1)[0].strip()
	temp = dice_string.split("d", 1)[1].strip()
	if temp.find("+") == -1:
		second = temp.strip()
		third = "0"
	else:
		second = temp.split("+", 1)[0].strip()
		third = temp.split("+", 1)[1].strip()
	new_game_dice[variable].append(int(first))
	new_game_dice[variable].append(int(second))
	new_game_dice[variable].append(int(third))


#Item arrays. Each item is a separate line in the array. Each line goes:
#name, type, quality, price, description
#type is 0-5 for equipment, 10-20 for items, and 99 for system items.
#If price is 0, the item is un sell/buy/drop able. (For story items.)
#Duplicate names are bad.
item = []

#Add an item to the item[] array
def additem(name, type, quality, price, description):
	item.append([])
	i = len(item)
	item[i-1].append(name)
	item[i-1].append(type)
	item[i-1].append(quality)
	item[i-1].append(price)
	item[i-1].append(description)

#takes an item name, and returns its location in the
#item[] array, with -1 == nonexisting.
def finditem(name):
	for i in range(len(item)):
		if name.lower() == item[i][0].lower():
			return i
	return -1


#read items.txt. This is called on startup.
def read_items():
	item_loc = g.mod_directory + "/data/items.txt"
	item_file = open(item_loc, 'r')

	#go through all lines of the file
	item_line = item_file.readline()
	while item_line != '':
		#strip out spaces/tabs
		item_line = item_line.strip()

		#ignore blank lines and comments
		if item_line[:1] == "#" or item_line[:1] == "":
			item_line = item_file.readline()
			continue

		#Connect lines with the "\" character.
		while item_line[-1:] == "\\":  #really a single backslash, BTW.
			item_line = item_line[:-1] + item_file.readline()[:-1] #for \n
		#split the line and clean up any remaining problems
		item_array = item_line.split("/")
		for i in range(5):
			item_array[i] = item_array[i].strip()

		#actually add the monster
		additem(item_array[0], int(item_array[1]),
			int(item_array[2]), int(item_array[3]), item_array[4])

		#ready another line
		item_line = item_file.readline()

	item_file.close()




#skills array. Each skill is a separate line in the array. Each line goes:
#name, effect, level, price, description, acquired
#effect is an integer that tells battle.py which case in a select to pick.
#level is the skillpoints required to get the skill,
#as well as the ep needed to use.
#acquired tells if the skill has already been learned by the player.
skill = []

#Add a skill to the skill[] array
def addskill(name, effect, level, price, description):
	skill.append([])
	i = len(skill)
	skill[i-1].append(name)
	skill[i-1].append(effect)
	skill[i-1].append(level)
	skill[i-1].append(price)
	skill[i-1].append(description)
	skill[i-1].append(0)


#takes a skill name, and returns its location in the
#skill[] array, with -1 == nonexisting.
def findskill(name):
	for i in range(len(skill)):
		if name.lower() == skill[i][0].lower():
			return i
	return -1

#gives the player a skill; takes the location in skill[] as input
def add_skill(skill_loc):
	if (skill[skill_loc][5] == 1):
		return 0
	else:
		skill[skill_loc][5] = 1
		return 1

#add skills. I really should change these to scripts sometime.
addskill("Rage",       0, 1, 10,
	"Gives you increased damage for the rest of a battle")
addskill("Sneak Away", 1, 1, 10,
	"Attempts to leave a battle.")
addskill("Dismember",  3, 2, 20,
	"Your next attack will do maximum damage, and ignore armor")
addskill("Frenzy",  2, 2, 30,
	"Your next attack will try to hit more than once")


#Monster array. Each monster is a separate line in the array. Each line goes:
#name, hp, attack, defense, XP on death, gold on death, description.
monster = []

#Add an monster to the monster[] array
def addmonster(mon_name, mon_hp, mon_attack, mon_defense, mon_xp,
			mon_gold, mon_desc):
	monster.append([])
	i = len(monster)
	monster[i-1].append(mon_name)
	monster[i-1].append(mon_hp)
	monster[i-1].append(mon_attack)
	monster[i-1].append(mon_defense)
	monster[i-1].append(mon_xp)
	monster[i-1].append(mon_gold)
	monster[i-1].append(mon_desc)


#find an appropriate monster for the dungeon level.
#-1 is returned if no monsters are available for this level;
#otherwise, the position in maps[zgrid].monster[] is returned.

def find_level_monster(level):
	if len(maps[level].monster) == 0: return -1
	mon_name = maps[level].monster[
		int(random() * len(maps[level].monster))]
	for i in range(len(monster)):
		if mon_name == monster[i][0]:
			return i
	print "monster " + mon_name + " not found in monster.txt."
	return -1

#read monster.txt. This is called on startup.
def read_monster():
	monster_loc = g.mod_directory + "/data/monster.txt"
	monster_file = open(monster_loc, 'r')

	#go through all lines of the file
	monster_line = monster_file.readline()
	while monster_line != '':
		#strip out spaces/tabs
		monster_line = monster_line.strip()

		#ignore blank lines and comments
		if monster_line[:1] == "#" or monster_line[:1] == "":
			monster_line = monster_file.readline()
			continue

		#split the line and clean up any remaining problems
		monster_array = monster_line.split("/")
		for i in range(6):
			monster_array[i] = monster_array[i].strip()

		#actually add the monster
		addmonster(monster_array[0], int(monster_array[1]),
			int(monster_array[2]), int(monster_array[3]),
				int(monster_array[4]), int(monster_array[5]), monster_array[6])

		#ready another line
		monster_line = monster_file.readline()

	monster_file.close()



#Rolls dice in the form 2d6, where 2 is the number of dice, and 6 the number of
#sides on each die. modify is the bonus given to each die. Use die_roll(2, 6)

#Modify is the bonus given to each die. Use die_roll(2, 6) + 4 for bonuses on
#the entire roll, die_roll(2, 6, 1) for bonuses on each roll. Default == 0
def die_roll(dice, sides, modfy = 0):
	if sides < 1:
		sides = 1
	if dice < 1:
		dice = 1

	sum = 0
	for x in range(dice):
		die = int(((random() * sides) + 1 + modfy))
		sum = sum + die

	return sum


#Returns the symbol of the given tile. X and Y are relative
#to the location of the player.
def checklocation(x, y):

	#transform from relative to absolute coordinates
	x = xgrid + x
	y = ygrid + y

	#Assume that all off-map areas are rock.
	if x < 0 or y < 0:
		return 'a'
	try:
		#this will fail when looking too far right or down
		return maps[zgrid].field[y][x].name
	except IndexError:
		#off the map, so rock.
		return 'a'

#like checklocation, but just returns 0 (rock) or 1 (grass)
def iswalkable(x, y):
	#transform from relative to absolute coordinates
	x = xgrid + x
	y = ygrid + y

	#Assume that all off-map areas are rock.
	if x < 0 or y < 0:
		return 0
	try:
		#this will fail when looking too far right or down
		if maps[zgrid].field[y][x].walk == 0:
			return 0
	except IndexError:
		return 0
	#By this point, it is known to be walkable.
	return 1

#takes the name of a map, and returns its zgrid.
def mapname2zgrid(name):
	for i in range(len(maps)):
		if maps[i].name == name:
			return i
	else:
		print "file " + name + " not found"
		return -1

#this loads the various tiles.
def load_tiles():
	global tiles
	tiles = {"blank" : ImageTk.PhotoImage("RGB", (32, 32))}

	for root, dirs, files in walk(g.mod_directory + "/images/tiles/"):
		(head, tail) = path.split(root)
		if (tail != "CVS"):
			for tilename in files:
			    #if image is in a sub-dir:
			    if (root != g.mod_directory + "/images/tiles/"):
				i = len(g.mod_directory + "/images/tiles/")
				tiles[root[i:] + "/" + tilename] = \
				    ImageTk.PhotoImage(file=root + "/" + tilename)
			    else: #if image is in root dir
				tiles[tilename] = \
				    ImageTk.PhotoImage(file=root + "/" + tilename)
